function CGS()

% This function is an example of tracker integration into the toolkit.
% The implemented tracker is a very simple NCC tracker that is also used as
% the baseline tracker for challenge entries.
%
% *************************************************************
% VOT: Always call exit command at the end to terminate Matlab!
% *************************************************************
cleanup = onCleanup(@() exit() );

addpath(genpath('E:\Projects\VOT\VOT2017\CGS\'));
% *************************************************************
% VOT: Set random seed to a different value every time.
% *************************************************************
RandStream.setGlobalStream(RandStream('mt19937ar', 'Seed', sum(clock)));
% **********************************
% VOT: Get initialization data
% **********************************
[handle, image, region] = vot('rectangle');

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
global opts iterEnergy
warning off all
%% initialize parameters
imgdat = initParameters_VOT(image, region);

%% initialize the tracker
[targetState, frSegment, models] = initTracker(imgdat, region);

frame = 1;
while(true)
    % **********************************
    % VOT: Get next frame
    % **********************************
    [handle, image] = handle.frame(handle);
    
    if(isempty(image))
        break;
    end
    frame = frame + 1;
    imgdat = imread(image);
    imgdat = gray2rgb(imgdat);    
  
    % set initial labeling of superpixels to all background
    [searchImage, searchPos, frSegment, newModels, labeling, neighbors, colorPotential] = initLabeling(imgdat, targetState, frSegment, models);
    preTargetState = targetState; % target state in the previous frame
    preTargetAppModel = newModels.targetAppModel;
    preModels = models;
    bestE = inf; % initialize the energy
    foreRatio = []; 
    backRatio = [];    
    % energy minimization
    for iter = 1:opts.maxIters
        labeling_ = labeling; % old labeling
        targetState_ = targetState; % old target state
        frSegment_ = frSegment; % old frame segment
        % obtain the labeling using fixed models and target state
        [partE, labeling, frSegment, affPotential] = calcLabeling(searchImage, frSegment, newModels, neighbors, colorPotential, foreRatio, backRatio, frame, iter);
        % obtain the target state using fixed models and labeling
        [targetState, newModels, targetE] = calcTargetState(imgdat, searchImage, preTargetState, frSegment, newModels, preTargetAppModel);        
        % if new energy worse (or same), restore previous labeling and models
        totalE = targetE + partE;
        if(totalE >= bestE)
            labeling = labeling_;
            targetState = targetState_;
            frSegment = frSegment_;
            models = fitModels;
            models.targetAppModel = preTargetAppModel;                        
            break;
        else
            bestE = totalE;
            iterEnergy(frame, iter) = bestE;  
        end
        preTargetAppModel = newModels.targetAppModel;
        [frSegment, fitModels, foreRatio, backRatio, extraNodes] = refitModels(imgdat, searchPos, targetState, labeling, frSegment, newModels, affPotential, preModels, partE, frame); % otherwise refit the models
        if(iter == opts.maxIters)
            models = fitModels;
            models.targetAppModel = preTargetAppModel;            
        end
        newModels = proposeStructModels(targetState, searchPos, frSegment, fitModels, extraNodes, frame); % update the target structure model
    end
    % update the target appearance and target structure model
    [frSegment, models] = updateModels(searchPos, labeling, targetState, frSegment, models, frame);
    
    region = targetState.targetPos;
    handle = handle.report(handle, region);
end

% **********************************
% VOT: Output the results
% **********************************
handle.quit(handle);

end