function [targetState, targetE, targetAppModel] = locatePos(imgdat, targetCenter, cfResponse, tsResponse, confidenceMap, targetAppModel)

global opts

% merge the response
mgResponse = opts.fusionWeight*cfResponse + (1-opts.fusionWeight)*tsResponse;
[row, col] = find(mgResponse == max(mgResponse(:)), 1);
center = (1 + targetAppModel.norm_delta_area) / 2;
targetCenter = targetCenter + ([row, col] - center)/targetAppModel.area_resize_factor;

% set the scale
im_patch_scale = getScaleSubwindow(imgdat, targetCenter, targetAppModel.base_target_sz, targetAppModel.scale_factor*targetAppModel.scale_factors, targetAppModel.scale_window, targetAppModel.scale_model_sz, targetAppModel.hog_scale_cell_size);
xsf = fft(im_patch_scale,[],2);
scale_response = real(ifft(sum(targetAppModel.sf_num .* xsf, 1) ./ (targetAppModel.sf_den + targetAppModel.lambda) ));
recovered_scale = ind2sub(size(scale_response),find(scale_response == max(scale_response(:)), 1));
scale_factor = targetAppModel.scale_factor*targetAppModel.scale_factors(recovered_scale);
if(scale_factor < targetAppModel.min_scale_factor)
    scale_factor = targetAppModel.min_scale_factor;
elseif(scale_factor > targetAppModel.max_scale_factor)
    scale_factor = targetAppModel.max_scale_factor;
end
targetAppModel.scale_factor = scale_factor;
% use new scale to update bboxes for target, filter
targetSize = round(targetAppModel.base_target_sz * targetAppModel.scale_factor);

if(opts.rotateMode)    
    targetPos = zeros(1, 8);
    targetPos(1) = targetCenter(2) - targetSize(2)/2;     
    targetPos(2) = targetCenter(1) + targetSize(1)/2; 
    targetPos(3) = targetCenter(2) - targetSize(2)/2;
    targetPos(4) = targetCenter(1) - targetSize(1)/2;
    targetPos(5) = targetCenter(2) + targetSize(2)/2; 
    targetPos(6) = targetCenter(1) - targetSize(1)/2;
    targetPos(7) = targetCenter(2) + targetSize(2)/2;
    targetPos(8) = targetCenter(1) + targetSize(1)/2;
else
    targetPos = [targetCenter([2,1]) - targetSize([2,1])/2, targetSize([2,1])];
end
targetState.targetPos = targetPos;
targetState.targetCenter = targetCenter;
targetState.targetSize = targetSize;

targetE = -max(mgResponse(:))*opts.targetWeight;

% show the response map
if(opts.showResponseMap)
    figure(3);
    subplot(2,2,1),imshow(confidenceMap,[]),title('confidence map');    
    subplot(2,2,2),imshow(tsResponse,[]),title('part response');
    subplot(2,2,3),imshow(cfResponse,[]),title('target response');
    subplot(2,2,4),imshow(mgResponse,[]),title('fusion response');
end